"""
Model exported as python.
Name : RpcuParing_stats_parcelles
Group : RPCUTools
With QGIS : 32600
"""

from qgis.core import QgsProcessing
from qgis.core import QgsProcessingAlgorithm
from qgis.core import QgsProcessingMultiStepFeedback
from qgis.core import QgsProcessingParameterVectorLayer
from qgis.core import QgsProcessingParameterFeatureSink
import processing


class Rpcuparing_stats_parcelles(QgsProcessingAlgorithm):

    def initAlgorithm(self, config=None):
        self.addParameter(QgsProcessingParameterVectorLayer('parcelles', 'parcelles_poly', types=[QgsProcessing.TypeVectorPolygon], defaultValue=None))
        self.addParameter(QgsProcessingParameterVectorLayer('vecteurs', 'parcelles_vecteurs_vec', types=[QgsProcessing.TypeVectorLine], defaultValue=None))
        self.addParameter(QgsProcessingParameterFeatureSink('Ko', 'ko', optional=True, type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=False, defaultValue=None))
        self.addParameter(QgsProcessingParameterFeatureSink('Parcelles_stats', 'parcelles_stats', optional=True, type=QgsProcessing.TypeVectorAnyGeometry, createByDefault=True, defaultValue='TEMPORARY_OUTPUT'))

    def processAlgorithm(self, parameters, context, model_feedback):
        # Use a multi-step feedback, so that individual child algorithm progress reports are adjusted for the
        # overall progress through the model
        feedback = QgsProcessingMultiStepFeedback(4, model_feedback)
        results = {}
        outputs = {}

        # Statistiques par catégories
        alg_params = {
            'CATEGORIES_FIELD_NAME': ['JOINT_VALUE'],
            'INPUT': parameters['vecteurs'],
            'VALUES_FIELD_NAME': 'longueur',
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['StatistiquesParCatgories'] = processing.run('qgis:statisticsbycategories', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(1)
        if feedback.isCanceled():
            return {}

        # Extraire par attribut
        alg_params = {
            'FIELD': 'NUM',
            'INPUT': parameters['parcelles'],
            'OPERATOR': 0,  # =
            'VALUE': '0',
            'FAIL_OUTPUT': QgsProcessing.TEMPORARY_OUTPUT,
            'OUTPUT': QgsProcessing.TEMPORARY_OUTPUT
        }
        outputs['ExtraireParAttribut'] = processing.run('native:extractbyattribute', alg_params, context=context, feedback=feedback, is_child_algorithm=True)

        feedback.setCurrentStep(2)
        if feedback.isCanceled():
            return {}

        # Joindre les attributs par valeur de champ
        alg_params = {
            'DISCARD_NONMATCHING': False,
            'FIELD': 'JOINT_VALUE',
            'FIELDS_TO_COPY': [''],
            'FIELD_2': 'JOINT_VALUE',
            'INPUT': outputs['ExtraireParAttribut']['FAIL_OUTPUT'],
            'INPUT_2': outputs['StatistiquesParCatgories']['OUTPUT'],
            'METHOD': 1,  # Prendre uniquement les attributs de la première entité correspondante (un à un)
            'PREFIX': '_',
            'NON_MATCHING': parameters['Ko'],
            'OUTPUT': parameters['Parcelles_stats']
        }
        outputs['JoindreLesAttributsParValeurDeChamp'] = processing.run('native:joinattributestable', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        results['Ko'] = outputs['JoindreLesAttributsParValeurDeChamp']['NON_MATCHING']
        results['Parcelles_stats'] = outputs['JoindreLesAttributsParValeurDeChamp']['OUTPUT']

        feedback.setCurrentStep(3)
        if feedback.isCanceled():
            return {}

        # Appliquer le style
        alg_params = {
            'INPUT': outputs['JoindreLesAttributsParValeurDeChamp']['OUTPUT'],
            'STYLE': 'parcelles_vecteurs_stats.qml'
        }
        outputs['AppliquerLeStyle'] = processing.run('native:setlayerstyle', alg_params, context=context, feedback=feedback, is_child_algorithm=True)
        return results

    def name(self):
        return 'RpcuParing_stats_parcelles'

    def displayName(self):
        return 'RpcuParing_stats_parcelles'

    def group(self):
        return 'RPCUTools'

    def groupId(self):
        return 'RPCUTools'

    def createInstance(self):
        return Rpcuparing_stats_parcelles()
